<?php

namespace onespace\tools\activeApi\models\sim;

use onespace\tools\activeApi\components\clients\SimDetailsClient;

class SimDetails extends \onespace\tools\activeApi\models\BaseActiveApiModel {
    public $id;
    public $msisdn;
    public $iccid;
    public $imsi;
    public $description;
    public $name;
    public $active;
    public $airtimeBalance;
    public $dataBalanceInMb;
    public $smsBalance;
    public $bundleSizeInMb;
    public $price;
    public $daysValid;
    public $created_at;
    public $updated_at;

    public function client(): SimDetailsClient {
        return new SimDetailsClient();
    }

    public function primaryKey(): string {
        return 'id';
    }

    public function baseEndpoints(): array {
        return ['v1' => 'v1/api'];
    }
}
