<?php

namespace onespace\tools\activeApi\components\clients\teams\text;

/**
 * Set a general style for the text block in a teams message
 *
 * @see https://adaptivecards.io/schemas/adaptive-card.json#TextBlockStyle
 */
enum TextStyle: string {
    case UNSET = "UNSET";
    case DEFAULT = "default";
    case HEADING = "heading";
}
