<?php

namespace onespace\tools\activeApi\components\clients;

use Yii;

class PeopleClient extends \onespace\tools\activeApi\components\clients\base\BaseOneSpaceClient {
    public function serverUrl(): string {
        return Yii::$app->params['serviceUrls']['people'];
    }
}
