<?php

namespace onespace\tools\activeApi\components\clients\teams\text;

/**
 * Set a text colour of a text block in a teams message
 *
 * @see https://adaptivecards.io/schemas/adaptive-card.json#Colors
 */
enum TextColour: string {
    case UNSET = "UNSET";
    /** Colour: White */
    case DEFAULT = "default";
    /** Colour: Black */
    case DARK = "dark";
    /** Colour: White */
    case LIGHT = "light";
    /** Colour: Blue */
    case ACCENT = "accent";
    /** Colour: Green */
    case GOOD = "good";
    /** Colour: Yellow */
    case WARNING = "warning";
    /** Colour: Red */
    case ATTENTION = "attention";
}
