<?php

namespace onespace\tools\activeApi\components\clients\teams;

/**
 * Required tooling for a Teams Message Component block class.
 */
interface TeamsMessageComponent {
    /** Converts this object into an array which will later be converted to the relevant JSON */
    public function build(): array;
}
